#!/bin/sh
#
# Sophos Anti-Virus for Unix - Enterprise Manager installation/update script
# Copyright (c) 2003 Sophos Plc, www.sophos.com
VAexit()
{
   
   exit $1
}
error()
{
   
   fecho 'Error:  ' $*
   VAexit 1
}
fecho()
{
   case "$verbose,$1" in
      [0-9],[0-9]) if [ $verbose -ge $1 ]; then shift; else return; fi;;
   esac
   echo $@
}
pathcontains()
{
  i=$IFS
  IFS=:
  for d in $1; do
    if [ "$d" = "$2" ]; then
      IFS=$i; return 1
     fi
  done
  IFS=$i
  return 0
}
initialise()
{
  checksum=                     
  eminstall_conf="/etc/eminstall.conf" 
  em_install_icheckd=
  filename=                     
  install_commands=
  lastline=
  keep=                         
  central_install_dir=
  cache_install_dir= 
  calc_cidchecksum=
  cid_checksum=
  cid_list="cidsync.lst"         
  sleeptime=30                   
  tab="	" 
                                 
                                 
  tmp_cidfile=/tmp/cidlist.$$$
  tmp_filelist=/tmp/filelist.$$$
  sav_ic_dir=
  sav_install_dir=
  sav_binary_dir=
  sav_lib_dir=
  sav_data_dir=
  sav_man_dir=
  verbose=0
  
  
  
  
  if [ -d /sbin ]; then
    pathcontains $PATH /sbin
    if [ $? -eq 0 ]; then
       PATH=$PATH:/sbin
    fi
  fi
  if [ -d /usr/sbin ]; then
    pathcontains $PATH /usr/sbin
    if [ $? -eq 0 ]; then
       PATH=$PATH:/usr/sbin
    fi
  fi
  if [ -d /usr/local/bin ]; then
    pathcontains $PATH /usr/local/bin
    if [ $? -eq 0 ]; then
       PATH=$PATH:/usr/local/bin
    fi
  fi
  if [ -d /usr/local/man ]; then
    pathcontains $MANPATH /usr/local/man
    if [ $? -eq 0 ]; then
       MANPATH=$MANPATH:/usr/local/man
    fi
  fi
}
help_text()
{
  fecho "Sophos Anti-Virus Enterprise Manager Installation Utility"
  fecho "Copyright (c) 2003 Sophos Plc, www.sophos.com"
  fecho
  fecho "Display this help text by using: eminstall.sh -h"
  fecho 
  fecho "This utility performs an Enterprise Manager installation or update by checking"
  fecho "to see if the central installation directory has been updated.  If it has,"
  fecho "then an updated version of Sophos Anti-Virus is installed." 
  fecho
  fecho "eminstall.sh should be set up so that it is run by a regular cron job. If"
  fecho "no changes have occurred to the central installation directory, then nothing"
  fecho "will be done."
  fecho
  fecho "Configuration file $eminstall_conf should be set up to tell eminstall.sh"
  fecho "where the central installation directory is, and where the local cache"
  fecho "directory is.  The central installation directory is a directory, probably"
  fecho "on a remote machine, to which Enterprise Manager will download the latest"
  fecho "updates of Sophos Anti-Virus.  The cache directory is a directory local to"
  fecho "this machine, which holds the last update of Sophos Anti-Virus."
  fecho
  fecho "You can also specify which directories SAV itself should be installed in,"
  fecho "by setting values in the $eminstall.conf file."
  fecho
  fecho "The layout of eminstall.conf is:-"
  fecho "EM install CID = <central installation directory for EM>"
  fecho "EM cache dir = <cache directory for EM>"
  fecho "SAV install dir=<SAV install directory prefix, e.g. /usr/local>"
  fecho "SAV bin dir=<SAV bin directory, relative to install dir, e.g. bin>"
  fecho "SAV library dir=<SAV library dir, relative to install dir, e.g. lib>"
  fecho "SAV virus dir=<SAV virus dir, relative to install dir, e.g. sav>"
  fecho "SAV man dir=<SAV man pages dire, relative to install dir, e.g. man>"
  fecho "Verbose=<verbose level>, 0 or 1>"
  fecho "Keep=<file 1 to be kept> <file 2 to be kept> etc."
  fecho
  fecho "E.g. the lines:-"
  fecho "SAV install dir=/usr/local"
  fecho "SAV bin dir=bin"
  fecho "would set the SAV install dir prefix to /usr/local, and the binary install"
  fecho "directory to /usr/local/bin.  These values are optional and will be set to"
  fecho "default values if not supplied.  You MUST, however specify the EM install CID"
  fecho "and the EM cache dir."
  exit 0
}
EM_installation()
{
  
  initialise
  
  if [ ${#} -gt 0 ]; then
     while [ "$1" != "" ]; do
       if [ "$1" = "-h" ]; then
         help_text
       elif [ "$1" = "-c" ]; then
         if [ "$2" ]; then
           eminstall_conf=$2
           shift
         fi
       elif [ "$1" = "-v" ]; then
         verbose=1
       else
         help_text
       fi
     shift
     done
  fi
  
  read_conf_file
  
  
  see_if_update_required
  
  
  
  update_em_cache
  
  
  
  remove_unwanted_files
  
  install_sav_for_unix
}
read_conf_file()
{
  
  
  if [ -r $eminstall_conf ]; then
    l=`egrep -i "^[ $tab]*em[ $tab]*install[ $tab]*cid" $eminstall_conf`
    test "$l" && central_install_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
  
    l=`egrep -i "^[ $tab]*em[ $tab]*cache[ $tab]*dir" $eminstall_conf`
    test "$l" && cache_install_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
  
    l=`egrep -i "^[ $tab]*sav[ $tab]*install[ $tab]*dir" $eminstall_conf`
    test "$l" && sav_install_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
  
    l=`egrep -i "^[ $tab]*sav[ $tab]*bin[a-z]*[ $tab]*dir" $eminstall_conf`
    test "$l" && sav_bin_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
  
    l=`egrep -i "^[ $tab]*sav[ $tab]*lib[a-z]*[ $tab]*dir" $eminstall_conf`
    test "$l" && sav_lib_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
  
    l=`egrep -i "^[ $tab]*sav[ $tab]*virus[ $tab]*[a-z]*[ $tab]*dir" $eminstall_conf`
    test "$l" && sav_data_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
  
    l=`egrep -i "^[ $tab]*sav[ $tab]*man[ $tab]*[a-z]*[ $tab]*dir" $eminstall_conf`
    test "$l" && sav_man_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
  
  
  
    l=`egrep -i "^[ $tab]*sav[ $tab]*ic[ $tab]*[a-z]*[ $tab]*dir" $eminstall_conf`
    test "$l" && sav_ic_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
    if [ ! "$sav_ic_dir" ]; then
      l=`egrep -i "^[ $tab]*sav[ $tab]*intercheck[ $tab]*[a-z]*[ $tab]*dir" $eminstall_conf`
      test "$l" && sav_ic_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
      if [ ! "$sav_ic_dir" ]; then
        l=`egrep -i "^[ $tab]*sav[ $tab]*icheckd[ $tab]*[a-z]*[ $tab]*dir" $eminstall_conf`
        test "$l" && sav_ic_dir="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
      fi
    fi
  
    l=`egrep -i "^[ $tab]*verbose" $eminstall_conf`
    test "$l" && v="`echo $l | sed -e 's/^.*= *//' -e 's/ *$//'`"
    if [ "$v" = "" ]; then
      verbose=0
    elif [ "$v" = "0" ]; then
      verbose=0
    else
      verbose=1
    fi
    l=`egrep -i "^[ $tab]*keep" $eminstall_conf`
    test "$l" && keep="`echo $l | sed -e 's/^.*= *//'`" 
  else
    error Cannot perform EM installation - no $eminstall_conf file found.  Run eminstall.sh -h for details 
  fi
  fecho 1 EM Central installation directory is $central_install_dir
  fecho 1 EM Cache directory is $cache_install_dir
}
see_if_update_required()
{
  if [ "$central_install_dir" = "" ]; then
     error Central install directory not set up
  fi
  if [ "$cache_install_dir" = "" ]; then
     error Cache install directory not set up
  fi
  if [ "$central_install_dir" = "$cache_install_dir" ]; then
     error Central install directory and cache install directory are the same
  fi
  if [ ! -d $central_install_dir ]; then
    
    error cannot find EM central installation directory $central_install_dir
  fi
  
  
  if [ ! -r $central_install_dir/$cid_list ]; then
    error Could not find EM installation file list $cid_list
  fi
  
  
  
  if [ ! -d $cache_install_dir ]; then
    
    mkdir $cache_install_dir
  fi
  
  
  
  if [ -r $cache_install_dir/$cid_list ]; then
     diff $central_install_dir/$cid_list $cache_install_dir/$cid_list > /dev/null 2> /dev/null
     if [ $? -eq 0 ]; then
        fecho 1 Central installation list is identical, no need to update.
        exit 0
     fi
  fi
  
  
  
  
  calcmd5=$cache_install_dir/calcmd5
  if [ -r $calcmd5 ]; then
     if [ ! -x $calcmd5 ]; then
        chmod u+x $calcmd5
      fi
  else 
    calcmd5=$central_install_dir/calcmd5
    if [ ! -r $calcmd5 ]; then
      error Could not locate checksum program, cannot proceed with EM installation
    elif [ ! -x $calcmd5 ]; then
      cp -f $calcmd5 $cache_install_dir
      calcmd5=$cache_install_dir/calcmd5
      chmod u+x $calcmd5
    fi
  fi
  
  
  
  lastline=`tail -1 $central_install_dir/$cid_list`
  cid_checksum=$lastline
  
  
  
  
  
  grep -v "$lastline" $central_install_dir/$cid_list > $tmp_cidfile 2> /dev/null
  calc_cidchecksumm=`$calcmd5 $tmp_cidfile`
  rm -rf $tmp_cidfile
  if [ "$calc_cidchecksum" != "$cidchecksum" ]; then
      error Checksum error with new central installation cid list - aborting
  fi
  if [ ! -r $cache_install_dir/$cid_list ]; then
     fecho 1 EM installation cache list not found, updating cache...
  fi
}
update_em_cache()
{
  
  fecho 1 Updating files from EM central installation directory cache...
  
  
  
  
  
  
  
  grep "icheckd" $central_install_dir/$cid_list | grep -v "icheckd.1" | grep -v "icheckd.conf.5" > /dev/null 2> /dev/null
  if [ $? -eq 0 ]; then
      em_install_icheckd=yes
  fi
  while read filename checksum ; do
  
  
  
  
  if [ "$filename" != "$cid_checksum" ]; then
     
     
     
     if [ -r $cache_install_dir/$filename ]; then
       calcdchecksum=`$calcmd5 $cache_install_dir/$filename`
       if [ "$calcdchecksum" = "$checksum" ]; then
         fecho 2 Checksum of $filename was same as central checksum file
         
         
         
         continue
       else
          fecho 1 Checksum of file $filename differs from that in EM list.
          fecho 1 $calculated checksum was $calcdchecksum, from list was $checksum
       fi
     fi
     
     
     
     
     
     if [ ! -r $central_install_dir/$filename ]; then
        error Could not find file $central_install_dir/$filename in $cid_list
     else
        
        
        
        
        
        
        
        fecho 1 Updating $filename from EM installation CID to cache...
        rm -f $cache_install_dir/$filename
        cp -f $central_install_dir/$filename $cache_install_dir/$filename
        
        if [ -r $cache_install_dir/$filename ]; then
          calcdchecksum=`$calcmd5 $cache_install_dir/$filename`
          if [ "$calcdchecksum" = "$checksum" ]; then
            
            continue
          fi
        fi
        
        
        
        
        
        
        fecho Checksum is incorrect on file $filename copied from CID - retrying copy...
        sleep $sleeptime
        rm -f $cache_install_dir/$filename
        cp -f $central_install_dir/$filename $cache_install_dir/$filename
        if [ -r $cache_install_dir/$filename ]; then
          calcdchecksum=`$calcmd5 $cache_install_dir/$filename`
          if [ "$calcdchecksum" = "$checksum" ]; then
            continue
          fi
        fi
        fecho Checksum is incorrect on file $filename copied from CID - final retry...
        sleep $sleeptime 
        rm -f $cache_install_dir/$filename
        cp -f $central_install_dir/$filename $cache_install_dir/$filename
        if [ -r $cache_install_dir/$filename ]; then
          calcdchecksum=`./calcmd5 $cache_install_dir/$filename`
          if [ "$calcdchecksum" = "$checksum" ]; then
            continue
          fi
        fi
        error Abandoning update - $filename and central checksum list do not correspond
      fi
    else
      
      
      
      
      
      rm -f $cache_install_dir/$cid_list
      cp -f $central_install_dir/$cid_list $cache_install_dir/$cid_list
      if [ -r $cache_install_dir/$cid_list ]; then
        fecho 1 Validating checksum on $cid_list...
        grep -v "$lastline" $cache_install_dir/$cid_list > $tmp_cidfile
        calcdchecksum=`$calcmd5 $tmp_cidfile`
        rm -rf $tmp_cidfile
        if [ "$calcdchecksum" != "$filename" ]; then
          error Checksum error - checksum of copied $cid_list is incorrect...
        fi
      else
        error $cid_list has not been copied into cache install dir...
      fi
    fi
  done < $central_install_dir/$cid_list
}
remove_unwanted_files()
{
  fecho 1 Removing files not required from EM install cache...
  cd $cache_install_dir
  if [ ! -r $cid_list ]; then
    return
  fi
  ls > $tmp_filelist
  while read filename; do
    grep "$filename" $cid_list > /dev/null 2> /dev/null
    
    
    if [ $? -ne 0 ]; then
      
      
      if [ "$filename" != "$cid_list" ]; then
        remove_file=yes
        
        if [ "$keep" != "" ]; then
          echo $keep | grep "$filename" > /dev/null 2> /dev/null
          if [ $? -eq 0 ]; then
            remove_file=no
          fi
        fi
        if [ "$remove_file" = "yes" ]; then
          fecho 1 Removing $filename - does not exist in $cid_list
          rm -f $filename
        fi
      fi
    fi
  done < $tmp_filelist
  rm -f $tmp_filelist
}
install_sav_for_unix()
{
  
  
  
  cd $cache_install_dir
  install_commands=""
  if [ "$sav_install_dir" != "" ]; then
    install_commands="$install_commands -d $sav_install_dir"
  fi
  if [ "$sav_bin_dir" != "" ]; then
    install_commands="$install_commands -b $sav_bin_dir"
  fi
  if [ "$sav_lib_dir" != "" ]; then
    install_commands="$install_commands -l $sav_lib_dir"
  fi
  if [ "$sav_data_dir" != "" ]; then
    install_commands="$install_commands -s $sav_data_dir"
  fi
  if [ "$sav_man_dir" != "" ]; then
    install_commands="$install_commands -m $sav_man_dir"
  fi
  
  
  if [ ! "$em_install_icheckd" ]; then
    install_commands="$install_commands -ni"
  else
    
    
    
    
    install_commands="$install_commands -ssi"
    if [ "$sav_ic_dir" != "" ]; then
      install_commands="$install_commands -i $sav_ic_dir"
    fi
  fi
  if [ $verbose -gt 1 ]; then
    install_commands="$install_commands -v"
  fi
  
  fecho Installing updated files from EM installation cache...
  
  chmod u+x sweep                      
  chmod u+x icheckd
  chmod u+x libsavi*
  
  chmod u+x install.sh                 
  
  
  ./install.sh $install_commands -em -ri
  
  if [ $? -ne 0 ]; then
     error "EM update worked, but installation failed"
  else
     fecho Installation successful
  fi
}
EM_installation $*
exit 0
